@@ -43,10 +43,8 @@
 #include <glib/gi18n.h>
 #include <gio/gio.h>
 #include <gnome.h>
-#include <libgnomeui/gnome-window-icon.h>
-#include <gtk/gtkbuilder.h>
+#include <gtk/gtk.h>
 #include <gdk-pixbuf/gdk-pixbuf.h>
-#include <libgnomecanvas/gnome-canvas-pixbuf.h>
 #include <libgnome/libgnometypebuiltins.h>
 #include <gdk/gdkx.h>
 #include <gdk/gdk.h>
@@ -58,11 +56,6 @@
 
 #include <libsoup/soup.h>
 
-#include <sys/types.h>
-#include <signal.h>
-
-#define d(x)
-
 #define USE_PROXY_KEY 	"/system/http_proxy/use_http_proxy"
 #define PROXY_HOST_KEY	"/system/http_proxy/host"
 #define PROXY_PORT_KEY	"/system/http_proxy/port"
@@ -73,22 +66,18 @@
 #define GTK_THEME_KEY	"/desktop/gnome/interface/gtk_theme"
 #define ICON_THEME_KEY  "/desktop/gnome/interface/icon_theme"
 #define DESKTOP_IS_HOME_DIR "/apps/nautilus/preferences/desktop_is_home_dir"
+#define MIN_REPORT_DETAILS_CHARS 10
 
 static GOptionData gopt_data;
 static int bug_count = 0;
 static GHashTable *apps = NULL;
 
 static const GOptionEntry options[] = {
-	{ "name",       '\0', 0, G_OPTION_ARG_STRING,   &gopt_data.name,          N_("Name of contact"),                         N_("NAME") },
-	{ "email",      '\0', 0, G_OPTION_ARG_STRING,   &gopt_data.email,         N_("Email address of contact"),                N_("EMAIL") },
 	{ "package",    '\0', 0, G_OPTION_ARG_STRING,   &gopt_data.package,       N_("Package containing the program"),          N_("PACKAGE") },
-	{ "package-ver",'\0', 0, G_OPTION_ARG_STRING,   &gopt_data.package_ver,   N_("Version of the package"),                  N_("VERSION") },
 	{ "appname",    '\0', 0, G_OPTION_ARG_FILENAME, &gopt_data.app_file,      N_("File name of crashed program"),            N_("FILE") },
 	{ "pid",        '\0', 0, G_OPTION_ARG_INT,      &gopt_data.pid,           N_("PID of crashed program"),                  N_("PID") },
-	{ "core",       '\0', 0, G_OPTION_ARG_FILENAME, &gopt_data.core_file,     N_("Core file from program"),                  N_("FILE") },
 	{ "include",    '\0', 0, G_OPTION_ARG_FILENAME, &gopt_data.include_file,  N_("Text file to include in the report"),      N_("FILE") },
 	{ "minidump",   '\0', 0, G_OPTION_ARG_FILENAME, &gopt_data.minidump_file, N_("MiniDump file with info about the crash"), N_("FILE") },
-	{ "kill",       '\0', 0, G_OPTION_ARG_INT,      &gopt_data.kill,          N_("PID of the program to kill after the report"), N_("KILL") },
 	{ NULL }
 };
 
@@ -353,6 +342,55 @@ save_email (const char *email)
 	g_object_unref (conf_client);
 }
 
+static void
+copy_link_item_activated_cb (GtkMenuItem *menu_item,
+			     const gchar *link)
+{
+	GtkClipboard *clipboard;
+
+	clipboard = gtk_clipboard_get (GDK_SELECTION_CLIPBOARD);
+	gtk_clipboard_set_text (clipboard, link, -1);
+}
+
+static GtkWidget *
+build_link_menu (const gchar *link)
+{
+	GtkWidget *menu;
+	GtkWidget *item;
+	GtkWidget *image;
+
+	menu = gtk_menu_new ();
+	item = gtk_image_menu_item_new_with_mnemonic (_("Copy _Link Address"));
+	image = gtk_image_new_from_stock (GTK_STOCK_COPY, GTK_ICON_SIZE_MENU);
+	gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (item), image);
+	gtk_widget_show_all (item);
+	g_signal_connect (item, "activate",
+			  G_CALLBACK (copy_link_item_activated_cb), (char *) link);
+	gtk_menu_shell_append (GTK_MENU_SHELL (menu), item);
+
+	return menu;
+}
+
+static gboolean
+link_button_press_event_cb (GtkWidget *widget,
+			    GdkEventButton *event,
+			    gpointer data)
+{
+	const gchar *link = gtk_link_button_get_uri (GTK_LINK_BUTTON (widget));
+
+	if (event->type == GDK_BUTTON_PRESS && event->button == 3) {
+		GtkWidget *popup;
+
+		popup = build_link_menu (link);
+		gtk_menu_popup (GTK_MENU (popup), NULL, NULL, NULL, NULL,
+				event->button, event->time);
+
+		return TRUE;
+	}
+
+	return FALSE;
+}
+
 static void
 link_callback (GtkLinkButton *button, gpointer user_data)
 {
@@ -410,12 +448,13 @@ network_error (SoupMessage *msg, GtkBuilder *ui)
 		
 	dialog = gtk_message_dialog_new_with_markup (NULL,
 						      GTK_DIALOG_MODAL,
-						      GTK_MESSAGE_QUESTION,
+						      GTK_MESSAGE_WARNING,
 						      GTK_BUTTONS_YES_NO,
-						      _("<span weight=\"bold\">Network Connection Error</span>\n"
-						      "Maybe no Network Connection available.\n"
-						      "Do you want to store this report until "
-						      "a Network Connection is available?"));
+						      _("There has been a network error while sending the report. "
+							"Do you want to save this report and send it later?"));
+	gtk_message_dialog_format_secondary_text (GTK_MESSAGE_DIALOG (dialog),
+						  "%s", _("Please ensure that your internet connection is active "
+							  "and working correctly."));
 	res = gtk_dialog_run (GTK_DIALOG (dialog));
 	gtk_widget_destroy (dialog);
 
@@ -538,6 +577,8 @@ bug_sent (SoupSession *session, SoupMessage *msg, GtkBuilder *ui)
 		/* create a clickable link to the bug report */
 		urlbutton = gtk_link_button_new (text);
 		g_signal_connect (G_OBJECT (urlbutton), "clicked", G_CALLBACK (link_callback), NULL);
+		g_signal_connect (G_OBJECT (urlbutton), "button-press-event",
+				  G_CALLBACK (link_button_press_event_cb), NULL);
 		gtk_box_pack_end (GTK_BOX (main_vbox), urlbutton, FALSE, FALSE, 0);
 
 		gtk_widget_show (urlbutton);
@@ -876,6 +917,7 @@ email_domain_is_valid (const char *domain)
 {
 	char **labels;
 	char **this_label;
+	gboolean retval = FALSE;
 	
 	/* If there is no domain, there are no domain labels and the domain is
 	 * not valid. */ 
@@ -884,16 +926,22 @@ email_domain_is_valid (const char *domain)
 
 	/* Split the domain on the dot to validate labels. */
 	labels = g_strsplit (domain, ".", 0);
+	if (g_strv_length (labels) == 1) {
+		/* the domain doesn't contain any dot: it's not valid */
+		goto out;
+	}
 
 	for (this_label = labels; *this_label; this_label++) {
 		if (!email_domain_label_is_valid (*this_label)) {
-			g_strfreev (labels);
-			return FALSE;
+			goto out;
 		}
 	}
 
+	retval = TRUE;
+
+out:
 	g_strfreev (labels);
-	return TRUE;
+	return retval;
 }
 
 /* Check for *simple* email addresses of the form user@host, with checks
@@ -946,21 +994,83 @@ on_send_clicked (GtkWidget *button, gpointer data)
 	GnomeVersionInfo *gnome_version;
 	GtkRequisition requisition;
 	GtkBuilder *ui = (GtkBuilder*) data;
+	GtkWidget *details;
+	int i;
 
 	app = g_object_get_data (G_OBJECT (ui), "app");
 	gnome_version = g_object_get_data (G_OBJECT (ui), "gnome-version");
 
-	/* hide the send button immediately so that the user can't click
-	 * it more than once (this will create multiple bugs) */
-	gtk_widget_hide (GTK_WIDGET (gtk_builder_get_object (ui, "send-button")));
+	details = GTK_WIDGET (gtk_builder_get_object (ui, "details-view"));
+	i = gtk_text_buffer_get_char_count (
+			gtk_text_view_get_buffer (GTK_TEXT_VIEW (details)));
 
-	gtk_widget_show (GTK_WIDGET (gtk_builder_get_object (ui, "progressbar")));
-	gtk_widget_hide (GTK_WIDGET (gtk_builder_get_object (ui, "final-box")));
-	gtk_widget_hide (GTK_WIDGET (gtk_builder_get_object (ui, "review-box")));
+	if (i < MIN_REPORT_DETAILS_CHARS) {
+		GtkWidget *dialog;
+		GtkWidget *button;
+		GtkWidget *icon;
 
-	gtk_widget_size_request (GTK_WIDGET (gtk_builder_get_object (ui, "main-window")), &requisition);
-	gtk_window_resize (GTK_WINDOW (gtk_builder_get_object (ui, "main-window")),
-			   requisition.width, requisition.height);
+		int i;
+
+		dialog = gtk_message_dialog_new (NULL, 
+				GTK_DIALOG_MODAL, 
+				GTK_MESSAGE_INFO,
+				GTK_BUTTONS_NONE,
+				_("The description you provided for the "
+				  "crash is very short. Are you sure you want "
+				  "to send it?"));
+
+		/* Secondary text */
+		gtk_message_dialog_format_secondary_text 
+			(GTK_MESSAGE_DIALOG (dialog),
+			 _("A short description is probably not of much help "
+			   "to the developers investigating your report. "
+			   "If you provide a better one, for instance "
+			   "specifying a way to reproduce the crash, the "
+			   "issue can be more easily resolved."));
+
+		/* Review button */
+		button = gtk_dialog_add_button (GTK_DIALOG (dialog),
+						_("_Review description"),
+						GTK_RESPONSE_CANCEL);
+		icon = gtk_image_new_from_stock 
+				(GTK_STOCK_EDIT, GTK_ICON_SIZE_BUTTON);
+		gtk_button_set_image (GTK_BUTTON (button), icon);
+		gtk_widget_show (button);
+
+		/* Send anyway button */
+		button = gtk_dialog_add_button (GTK_DIALOG (dialog),
+						_("_Send anyway"),
+						GTK_RESPONSE_OK);
+		icon = gtk_image_new_from_stock 
+				(GTK_STOCK_OK, GTK_ICON_SIZE_BUTTON);
+		gtk_button_set_image (GTK_BUTTON (button), icon);
+		gtk_widget_show (button);
+
+		gtk_dialog_set_default_response (GTK_DIALOG (dialog), 
+						 GTK_RESPONSE_OK);
+		
+		i = gtk_dialog_run (GTK_DIALOG (dialog));
+
+		gtk_widget_destroy (dialog);
+
+		if (i != GTK_RESPONSE_OK)
+			return;
+	}
+
+	gtk_widget_show (GTK_WIDGET (
+			gtk_builder_get_object (ui, "progressbar")));
+	gtk_widget_hide (GTK_WIDGET (
+			gtk_builder_get_object (ui, "final-box")));
+	gtk_widget_hide (GTK_WIDGET (
+			gtk_builder_get_object (ui, "review-box")));
+
+	gtk_widget_size_request (GTK_WIDGET (
+			gtk_builder_get_object (ui, "main-window")), 
+			&requisition);
+
+	gtk_window_resize (GTK_WINDOW (
+			gtk_builder_get_object (ui, "main-window")),
+			requisition.width, requisition.height);
 			   
 	send_report (app, gnome_version, ui);
 }
@@ -1705,14 +1815,18 @@ main (int argc, char *argv[])
 		gtk_builder_set_translation_domain (ui, GETTEXT_PACKAGE);
 	}
 
-	if (!ui) {
-		buddy_error (NULL, 
+	g_free (s);
+
+	if (!ui || err) {
+		buddy_error (NULL,
 			     _("Bug Buddy could not load its user interface file.\n"
 			       "Please make sure Bug Buddy was installed correctly."));
+		if (err) {
+			g_error_free (err);
+		}
 		g_object_unref (program);
 		return 0;
 	}
-	g_free (s);	
 
 	main_window = GTK_WIDGET (gtk_builder_get_object (ui, "main-window"));
 	g_signal_connect (main_window, "delete-event", G_CALLBACK (delete_callback), ui);
